################################################################
#
#   Assembly Line - Company
#
################################################################

from albow.dialogs import alert
from imath import iceil
from factory import Factory
from part import create_material
from imath import iround, iceil

initial_cash = 20000
#base_freight_cost = 20
#freight_rate = 0.003

#------------------------------------------------------------------------

class MaterialStock(object):
	#  material    Material
	#  sheet_size  (width, height) sorted
	#  quantity    int
	
	quantity = 0
	
	def __init__(self, material, size):
		self.material = material
		self.sheet_size = size
	
	def sheet_cost(self):
		return self.material.sheet_cost(self.sheet_size)

#------------------------------------------------------------------------

class ProductHistory(object):
	#  number_sold     int
	#  number_trashed  int
	#  revenue         int
	#  #freight_cost    int
	
	number_sold = 0
	number_trashed = 0
	revenue = 0
	#freight_cost = 0

#------------------------------------------------------------------------

class Company(object):
	#
	#  factories            [Factory]
	#  cash                 int
	#  income               int
	#  expenditure          int
	#  capital_outlay       int
	#  machines_in_storage  {MachineType: [Machine]}
	#  material_stock       {(rgba, sorted_size): MaterialStock}
	#  material_stock_list  [MaterialStock]
	#  current_factory      Factory
	#  transactions         [(description, amount)]
	#  product_history      {key: ProductHistory}
	
	current_factory = None
	income = 0
	expenditure = 0
	capital_outlay = 0

	def __init__(self):
		self.factories = []
		self.cash = initial_cash
		self.machines_in_storage = {}
		self.material_stock = {}
		self.material_stock_list = []
		self.transactions = []
		self.product_history = {}
		self.new_factory()
	
	def __setstate__(self, d):
		self.__dict__.update(d)
		if 'material_stock' not in d:
			self.material_stock = {}
		if 'material_stock_list' not in d:
			self.material_stock_list = self.material_stock.items()
		if 'product_history' not in d:
			self.product_history = {}
	
	def get_profit(self):
		return self.income - self.expenditure
	
	profit = property(get_profit)
	
	def get_balance_of_earnings(self):
		return self.income - self.adjusted_expenditure
	
	balance_of_earnings = property(get_balance_of_earnings)
	
	def get_adjusted_expenditure(self):
		return self.expenditure - self.capital_outlay
	
	adjusted_expenditure = property(get_adjusted_expenditure)
	
	def new_factory(self):
		f = Factory()
		self.factories.append(f)
		self.select_factory(f)
	
	def select_factory(self, f):
		self.current_factory = f

	def put_machine_into_storage(self, machine):
		key = machine.__class__
		store = self.machines_in_storage.get(key)
		if not store:
			store = []
			self.machines_in_storage[key] = store
		store.append(machine)
		print "Store now contains", len(store), key.__name__ ###

	def get_machine_from_storage(self, machine_type, match = None):
		store = self.machines_in_storage.get(machine_type)
		if store:
			if match:
				for machine in store:
					if match(machine):
						print "Got a matching", machine_type.__name__, "from store" ###
						return machine
			else:
				print "Getting a", machine_type.__name__, "from store" ###
				return store.pop()
	
	def get_material_sheet_from_stock(self, material, size, order_quantity):
		#print "Getting a sheet of", material.formula(), size, "from stock" ###
		stock = self.find_material_stock(material, size)
		if not stock.quantity > 0 and order_quantity > 0:
			self.purchase_stock(stock, order_quantity)
		if stock.quantity > 0:
			sheet = create_material(material, size)
			stock.quantity -= 1
			#print "There are now", stock.quantity, "sheets left in stock" ###
			return sheet
		#else:
			#print "There are no sheets left in stock" ###
	
	def find_material_stock(self, material, size):
		sorted_size = list(size)
		sorted_size.sort()
		sorted_size = tuple(sorted_size)
		key = (material.rgba(), sorted_size)
		stock = self.material_stock.get(key)
		if not stock:
			stock = MaterialStock(material, sorted_size)
			self.material_stock[key] = stock
			self.material_stock_list.append(stock)
		return stock
	
	def purchase_stock(self, stock, quantity):
		cost = iceil(quantity * stock.sheet_cost())
		material = stock.material
		print "Company.purchase_stock: stock.sheet_size =", stock.sheet_size ###
		size = "%sx%s" % stock.sheet_size
		matdesc = "%s %s" % (material.formula(), size)
		desc = "Material purchase (%s)" % matdesc
		if cost <= self.cash:
			self.spend_cash(desc, cost)
			stock.quantity += quantity
		else:
			alert("Not enough cash to purchase %s sheets of %s "
				"(cost $%s, available funds $%s)" % (quantity, matdesc, cost, self.cash))

	def spend_cash(self, description, amount):
		assert isinstance(amount, (int, long))
		self.transactions.append((description, -amount))
		self.cash -= amount
		self.expenditure += amount
		print "Spent: %s $%s leaving $%s" % (description, amount, self.cash) ###

	def earn_cash(self, description, amount):
		assert isinstance(amount, (int, long))
		self.transactions.append((description, amount))
		self.cash += amount
		self.income += amount
		print "Earned: %s $%s leaving $%s" % (description, amount, self.cash) ###

	def begin_frame(self):
		factory = self.current_factory
		if factory:
			factory.begin_frame()

	def find_product_history(self, product):
		history = self.product_history.get(product.key)
		if not history:
			history = ProductHistory()
			self.product_history[product.key] = history
		return history
	
	def sell_products(self, product, crate):
		title = product.title
		num_sold = 0
		num_trashed = 0
		revenue = 0
		for error in crate:
			fraction_of_full_price = max(0.0, 1.0 - error / 0.05)
			if fraction_of_full_price > 0.0:
				price = iround(fraction_of_full_price * product.price)
				#print "Company: Selling a %s with error %s for $%s" % (title, error, price) ###
				num_sold += 1
				revenue += price
			else:
				#print "Company: Trashing a %s with error %s" % (title, error) ###
				num_trashed += 1
		#bulk = num_sold * product.area()
		#freight_cost = base_freight_cost + iceil(bulk * freight_rate)
		print "Company: Revenue = $%s" % revenue ###
		#print "Company: Freight cost = $%s" % freight_cost ###
		history = self.find_product_history(product)
		history.number_sold += num_sold
		history.number_trashed += num_trashed
		history.revenue += revenue
		#history.freight_cost += freight_cost
		#self.spend_cash("%s Freight Cost" % title, freight_cost)
		if revenue <> 0:
			self.earn_cash("%s Sales" % title, revenue)
