################################################################
#
#   Assembly Line - Game
#
################################################################

import os
from tempfile import mkstemp
from pickle import Pickler, Unpickler
import settings
from company import Company
from product import load_products

class Game(object):
	#
	#  company                Company
	#  selected_machine_type  MachineType
	#  factory_tool           string
	#  products               [Product]
	#  selected_product       Product
	
	company = None
	selected_machine_type = None
	factory_tool = 'select'
	save_dir = None
	save_name = None
	selected_product = None
	
	def __init__(self):
		products = load_products()
		self.products = products
		map = {}
		for product in products:
			map[product.key] = product
		self.products_by_key = map

	def save_game_enabled(self):
		return self.game_in_progress()

	def resume_game_enabled(self):
		return self.game_in_progress()

	def game_in_progress(self):
		return self.company is not None

	def new_game(self):
		self.company = Company()
		self.save_name = None
	
	def load_game(self, path):
		f = open(path, "rb")
		self.check_save_magic(f, settings.app_magic)
		self.check_save_magic(f, settings.save_file_magic)
		p = Unpickler(f)
		file_version = p.load()
		app_version = p.load()
		self.check_save_version(file_version, app_version)
		company = p.load()
		f.close()
		self.company = company
		self.set_save_path(path)
	
	def check_save_magic(self, f, expected):
		magic = f.read(len(expected))
		if magic <> expected:
			raise EnvironmentError("Does not seem to be a saved game file.")
	
	def check_save_version(self, file_version, app_version):
		if file_version > settings.save_file_version:
			this_version = "%s.%s.%s" % (settings.app_version,)
			raise EnvironmentError("Unrecognised save file format %s "
				"(saved by version %s)." % (file_version, this_version))
	
	def save_game(self, path):
		fd, tmp_path = mkstemp(dir = os.path.dirname(path))
		f = os.fdopen(fd, "wb")
		f.write(settings.app_magic)
		f.write(settings.save_file_magic)
		p = Pickler(f, 2)
		p.dump(settings.save_file_version)
		p.dump(settings.app_version)
		p.dump(self.company)
		f.close()
		os.rename(tmp_path, path)
		self.set_save_path(path)
	
	def set_save_path(self, path):
		self.save_dir, self.save_name = os.path.split(path)

	def factory(self):
		return self.company.current_factory
	
	def get_save_dir(self):
		return self.save_dir or settings.get_default_save_dir()
	
	def get_save_path(self):
		dir = self.get_save_dir()
		name = self.save_name or settings.save_file_default_name
		return os.path.join(dir, name)

	def begin_frame(self):
		company = self.company
		if company:
			company.begin_frame()

	def factory_running(self):
		company = self.company
		if company:
			return company.current_factory.running
	
	def product_with_key(self, key):
		return self.products_by_key.get(key)

#------------------------------------------------------------------------

game = Game()
