################################################################
#
#   Assembly Line - Packing Station
#
################################################################

from pygame import Rect
from albow.resource import get_image, get_font
from albow.controls import Label, Row, Grid, IntEditor
from machine import Machine, MachineNotConfiguredError
from part import compare_parts
from game import game
from controls import MachineEditor, ProductKeyField

#------------------------------------------------------------------------

class PackingStationEditor(MachineEditor):

	def create_contents(self, machine):
		return Grid([
			[Label("Product"),
				ProductKeyField(attribute = 'product_key', model = machine)],
			[Label("Crate Size"),
				Row([CrateSizeField(machine), Label("items")])],
		])

def CrateSizeField(machine):
	return IntEditor(min = 1, max = 100, attribute = 'crate_size', model = machine)

#------------------------------------------------------------------------

class PackingStation(Machine):
	#  product_key     string
	#  crate_size      int
	#  crate           [error]

	title = "Packing Station"
	cost = 200
	layer = 0
	icon = get_image("packing_station_icon.png")
	image = get_image("packing_station.png")
	editor_class = PackingStationEditor

	local_bounding_rect = Rect(-32, -32, 64, 96)

	local_covered_rects = [
		[local_bounding_rect],
		[]
	]
	
	#local_rejection_point = (0, 64)

	_product_key = None
	crate_size = 10
	
	def __init__(self):
		self.crate = []
	
	def __setstate__(self, d):
		if 'product_key' in d:
			d['_product_key'] = d.pop('product_key')
		self.__dict__.update(d)
		if 'crate' not in d:
			self.crate = []
	
	def get_product_key(self):
		return self._product_key
	
	def set_product_key(self, key):
		if key <> self._product_key:
			self.sell_crate()
			self._product_key = key
	
	product_key = property(get_product_key, set_product_key)
	
	def check_ready(self):
		if not self.product_key or not self.crate_size:
			raise MachineNotConfiguredError
	
	def draw_local(self, surf, t):
		org = self.local_topleft()
		t.draw_image(surf, [self.image], org)

	def handle_part(self, factory, part):
		#print "PackingStation.handle_part:", part ###
		product = game.product_with_key(self.product_key)
		if product:
			error = compare_parts(part, product.part)
			#print "PackingStation.handle_part: Error =", error ###
			factory.remove_part(part)
			crate = self.crate
			crate.append(error)
			if len(crate) >= self.crate_size:
				#game.company.sell_products(product, crate)
				#del crate[:]
				self.sell_crate()
		return True
	
	def sell_crate(self):
		crate = self.crate
		if crate:
			product = game.product_with_key(self.product_key)
			game.company.sell_products(product, crate)
			del crate[:]
	
#	def reject_part(self, factory, part):
#		p = self.global_point(self.local_rejection_point)
#		factory.move_part_to(part, p)
