################################################################
#
#   Assembly Line - Machines page
#
################################################################

from pygame import Rect
from albow.resource import get_font
from albow.palette_view import PaletteView
from screen import ALScreen
from machines import machine_types

palette_margin = 16

icon_size = (96, 48)
icon_margin = 8
icon_label_height = 16
icon_label_font = get_font(10, "Vera.ttf")

class MachinesPage(ALScreen):

	fg_color = (0, 0, 0)
	bg_color = (103, 177, 109)

	def __init__(self, shell):
		ALScreen.__init__(self, shell)
		pm = palette_margin
		r = Rect(pm, pm, self.width - 2 * pm, self.height - 2 * pm)
		self.machines_view = MachinesView(r, self.game)
		self.add(self.machines_view)


class MachinesView(PaletteView):

	fg_color = (0, 0, 0)

	def __init__(self, rect, game):
		self.game = game
		iw, ih = icon_size
		cw = iw + 2 * icon_margin
		ch = ih + 2 * icon_margin + icon_label_height
		ncols = rect.width // cw
		nrows = rect.height // ch
		PaletteView.__init__(self, (cw, ch), nrows, ncols, scrolling = True)
		self.topleft = rect.topleft
	
	def factory(self):
		return self.game.factory()

	def num_items(self):
		return len(machine_types)
	
	def click_item(self, n, e):
		mtype = machine_types[n]
		self.game.selected_machine_type = mtype
	
	def item_is_selected(self, n):
		return machine_types[n] is self.game.selected_machine_type

	def draw_item(self, surf, n, rect):
		mtype = machine_types[n]
		icon = mtype.icon
		x, y = rect.midtop
		r = icon.get_rect()
		r.center = x, y + icon_margin + icon_size[1] // 2
		surf.blit(icon, r)
		buf = icon_label_font.render(mtype.title, True, self.fg_color)
		r = buf.get_rect()
		x = rect.centerx
		y = rect.bottom - icon_margin
		r.midbottom = (x, y)
		surf.blit(buf, r)
