################################################################
#
#   Assembly Line - Products
#
################################################################

import os, sys
from albow.resource import resource_path
from part import Part

class Product(object):
	#  key       string
	#  title     string
	#  part      Part
	#  palette   [color]
	#  price     int
	#  description  string
	
	description = ""
	
	def area(self):
		w, h = self.part.pixels.get_size()
		return w * h

#----------------------------------------------------------------------------

class ProductParser(object):

	def __init__(self, f, path):
		self.f = f
		self.path = path
		self.lineno = 0
		self.eof = False
		self.next_line()
	
	def next_line(self):
		line = self.f.readline()
		if line:
			self.lineno += 1
			line = line.strip()
			chunks = line.split(None, 1)
			self.keyword = chunks[0]
			if len(chunks) > 1:
				self.data = chunks[1]
			else:
				self.data = ""
		else:
			self.eof = True
	
	def args(self):
		return self.data.split()
	
	def get_args(self, types):
		args = self.args()
		if len(args) <> len(types):
			self.error("Wrong number of args, expected %s got %s" % (
				len(types), len(args)))
		return [t(a) for (t, a) in zip(types, args)]
	
	def expect_data(self, kw):
		self.check_keyword(kw)
		result = self.data
		self.next_line()
		return result
	
	def expect_args(self, kw, types):
		self.check_keyword(kw)
		result = self.get_args(types)
		self.next_line()
		return result
	
	def expect_arg(self, kw, type):
		return self.expect_args(kw, [type])[0]
	
	def check_keyword(self, kw):
		if self.keyword <> kw:
			self.error("Expected '%s'" % kw)
	
	def color(self, name):
		try:
			return self.palette[name]
		except KeyError:
			self.error("Color '%s' not defined" % name)
	
	def read_product(self):
		description = []
		key = self.expect_data("part")
		title = self.expect_data("title")
		price = self.expect_arg("price", int)
		while self.keyword == "desc":
			description.append(self.data)
			self.next_line()
		size = self.expect_args("size", [int, int])
		part = Part(size)
		self.palette = {}
		while not self.eof:
			kw = self.keyword
			if kw == "color":
				name, r, g, b, a = self.get_args([str, int, int, int, int])
				self.palette[name] = (r * 0x11, g * 0x11, b * 0x11, a * 0x11)
				self.next_line()
			elif kw == "rect":
				c, l, t, w, h = self.get_args([str, float, float, float, float])
				part.fill_float_rect(self.color(c), (l, t, w, h))
				self.next_line()
			elif kw == "circle":
				c, x, y, r = self.get_args([str, float, float, float])
				part.fill_float_circle(self.color(c), (x, y), r)
				self.next_line()
			else:
				self.error("Unknown keyword '%s'" % kw)
		product = Product()
		product.key = key
		product.title = title
		product.price = price
		product.description = " ".join(description)
		product.part = part
		product.palette = [v for v in self.palette.values() if v[3]]
		return product
	
	def error(self, mess):
		raise EnvironmentError("%s, line %s: %s" % (
			self.path, self.lineno, mess))

#----------------------------------------------------------------------------

def load_product(path):
	#print "Loading product:", path ###
	f = open(path)
	parser = ProductParser(f, path)
	product = parser.read_product()
	f.close()
	return product

def load_products():
	dir = resource_path("products")
	filenames = os.listdir(dir)
	filenames.sort()
	products = []
	for filename in filenames:
		if filename.endswith(".prd"):
			try:
				path = os.path.join(dir, filename)
				products.append(load_product(path))
			except EnvironmentError, e:
				sys.stderr.write("%s\n" % e)
	return products
