################################################################
#
#   Assembly Line - Product view
#
################################################################

import textwrap
from pygame import Rect
from albow.widget import Widget
from albow.controls import Label, Row, Column, ValueView
from albow.palette_view import PaletteView
from albow.resource import get_font
from material import draw_formula
from part_view import PartInfoView, PixelsView

desc_font = get_font(12, "Vera.ttf")

class ProductView(Widget):

	def __init__(self, height, pixel_size, get_product, **kwds):
		Widget.__init__(self, **kwds)
		self.get_product = get_product
		pixels_view = PixelsView(pixel_size, self.get_part)
		info_view = PartInfoView(pixels_view.width, self.get_part)
		desc_view = DescView(get_product)
		material_view = ProductMaterialView(height, get_product)
		desc_view.width = pixels_view.width
		on_left = Column([desc_view, info_view, pixels_view], align = 'l')
		on_right = Column([Label("Materials"), material_view], align = 'l')
		self.add(Row([on_left, on_right], align = 't'))
		self.shrink_wrap()
	
	def get_part(self):
		product = self.get_product()
		if product:
			return product.part


class DescView(Label):

	def __init__(self, get_product):
		Label.__init__(self, "", width = 400, font = desc_font)
		self.height = 30
		self.get_product = get_product
	
	def get_text(self):
		prod = self.get_product()
		if prod:
			text = prod.description
			return textwrap.fill(text, 80)
		else:
			return ""
	
	def set_text(self, x):
		pass

	text = property(get_text, set_text)


#def DescView(get_product):
#	def get():
#		prod = get_product()
#		if prod:
#			return prod.description
#		else:
#			return ""
#	return ValueView(width = 400, height = 60, font = desc_font, get_value = get)


class ProductMaterialView(PaletteView):

	column_width = 100
	cell_height = 20
	#border_width = 2

	def __init__(self, height, get_product):
		self.get_product = get_product
		nrows = height // self.cell_height
		cell_size = (self.column_width, self.cell_height)
		PaletteView.__init__(self, cell_size, nrows, 1, scrolling = True)
	
	def num_items(self):
		prod = self.get_product()
		if prod:
			return len(prod.palette)
		else:
			return 0
	
	def item_is_selected(self, n):
		return False
	
	def draw_item(self, surf, n, rect):
		s = self.cell_height
		color = self.get_product().palette[n]
		r = Rect(2, 2, s - 2, s - 2)
		r.move_ip(rect.topleft)
		surf.fill(color, r)
		draw_formula(surf, color, (r.left + s + 5, r.top + 2), self.fg_color)

		