################################################################
#
#   Assembly Line - Products page
#
################################################################

from albow.controls import Label, Row, Column
from albow.palette_view import PaletteView
from screen import ALScreen
from product_view import ProductView

class ProductsPage(ALScreen):

	bg_color = (255, 244, 155)
	fg_color = (0, 0, 0)

	def __init__(self, shell):
		ALScreen.__init__(self, shell)
		height = self.height - 24
		list_view = ProductListView(self.game, height - 32)
		product_view = ProductView(list_view.height,
			pixel_size = 8,
			get_product = self.get_product,
			get_visible = self.product_selected)
		contents = Row([list_view, product_view], align = 't')
		contents.topleft = (16, 16)
		self.add(contents)
	
	def product_selected(self):
		return self.game.selected_product is not None
	
	def get_product(self):
		return self.game.selected_product


class ProductListView(PaletteView):

	cell_width = 150
	lmargin = 5
	highlight_style = 'fill'
	fg_color = (0, 0, 0)
	sel_color = (62, 188, 254)
	border_width = 2

	def __init__(self, game, height):
		self.game = game
		cw = self.cell_width
		ch = self.font.get_linesize() + 4
		nr = height // ch
		PaletteView.__init__(self, (cw, ch), nr, 1, scrolling = True)
	
	def num_items(self):
		return len(self.game.products)
	
	def item_is_selected(self, n):
		game = self.game
		return game.products[n] is game.selected_product
	
	def click_item(self, n, e):
		game = self.game
		game.selected_product = game.products[n]
	
	def draw_item(self, surf, n, rect):
		product = self.game.products[n]
		buf = self.font.render(product.title, True, self.fg_color)
		r = buf.get_rect()
		r.midleft = rect.left + self.lmargin, rect.centery
		surf.blit(buf, r)
