################################################################
#
#   Assembly Line - Shell
#
################################################################

import os, sys
from albow.shell import Shell
from albow.dialogs import alert, ask
from albow.file_dialogs import request_old_filename, request_new_filename
from game import game
from menu_screen import MenuScreen
from play_screen import PlayScreen
import settings

class ALShell(Shell):

	bg_color = (224, 224, 224)

	def __init__(self, display):
		Shell.__init__(self, display)
		self.redraw_every_frame = False
		self.set_timer(settings.frame_time)
		self.game = game
		self.menu_screen = MenuScreen(self)
		self.play_screen = PlayScreen(self)
		options = settings.options
		self.show_menu()
		game.new_game()
		if options.filename:
			self.load_game_file(options.filename)
		if options.dbg_company_page:
			self.show_company_page()
		if options.dbg_products_page:
			self.show_products_page()
		if options.dbg_factory_page:
			self.show_factory_page()
		if options.dbg_machines_page:
			self.show_machines_page()
		if options.dbg_inspection_page:
			self.show_inspection_page()
		if options.dbg_stats_page:
			self.show_stats_page()
	
	def show_menu(self):
		self.show_screen(self.menu_screen)

	def show_play_screen(self):
		self.show_screen(self.play_screen)
	
	def show_company_page(self):
		self.show_play_screen()
		self.play_screen.show_company_page()

	def show_products_page(self):
		self.show_play_screen()
		self.play_screen.show_products_page()

	def show_factory_page(self):
		self.show_play_screen()
		self.play_screen.show_factory_page()
	
	def show_machines_page(self):
		self.show_play_screen()
		self.play_screen.show_machines_page()
	
	def show_inspection_page(self):
		self.show_play_screen()
		self.play_screen.show_inspection_page()
	
	def show_stats_page(self):
		self.show_play_screen()
		self.play_screen.show_stats_page()
	
	def new_game(self):
		self.game.new_game()
		self.show_company_page()
	
	def load_game(self):
		dir = self.game.get_save_dir()
		path = request_old_filename(suffixes = [settings.save_file_suffix],
			directory = dir)
		if path:
			self.load_game_file(path)
	
	def load_game_file(self, path):
		try:
			self.game.load_game(path)
			self.show_company_page()
			if settings.options.dbg_add_cash:
				self.game.company.cash += 20000
		except EnvironmentError, e:
			alert("Unable to load '%s': %s" % (
				os.path.basename(path), e))
	
	def save_game(self):
		game = self.game
		old_path = game.get_save_path()
		new_path = request_new_filename(prompt = "Save game as:",
			suffix = settings.save_file_suffix, pathname = old_path)
		if new_path:
			try:
				game.save_game(new_path)
			except EnvironmentError, e:
				alert("Unable to save '%s': %s" % (
					os.path.basename(new_path), mess))
	
	def resume_game(self):
		self.show_play_screen()

	def confirm_quit(self):
		return ask("Quit the game?") == "OK"

	def begin_frame(self):
		game = self.game
		game.begin_frame()
		screen = self.current_screen
		if screen:
			screen.begin_frame()
		if game.factory_running():
			self.invalidate()
